#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>

void fatal ( char *comment ) {
	perror(comment);
	exit(0);
}

double jetzt () {
	struct timeval t;
	gettimeofday(&t,NULL);
	double ergebnis = (double) t.tv_sec * 1000.0 + ((double) t.tv_usec / 1000.0);
	return ergebnis;
}

int main ( int ac, char **av ) {
	// Argumente lesen
	if (ac != 5) {
		fprintf(stderr,"usage: %s <n messages> <size> <host> <port>\n",av[0]);
		exit(0);
	}
	long n_messages = atol(av[1]);
	long message_size = atol(av[2]);
	char *host = av[3];
	long port = atol(av[4]);

 	// Socket erzeugen
	int s = socket(PF_INET,SOCK_DGRAM,0);
	if (s == -1) fatal("socket");

	struct sockaddr_in addr;
	memset((char *) &addr,0,sizeof(addr));
	addr.sin_family = PF_INET;
        struct hostent *he = gethostbyname(host);
        if (he == 0) fatal("gethostbyname");
        memcpy(&(addr.sin_addr),he->h_addr_list[0],sizeof(struct in_addr));
        addr.sin_port = htons(port);                                           	unsigned long inaddr_any = INADDR_ANY;
	
	// Hinreichend grossen Puffer bereitstellen
	char *buffer = new char[message_size];
	for (int i=0; i<message_size; i++) buffer[i] = i % 256;
	char ack[4];

	// Vorgegebene Anzahl an Nachrichten versenden
	long n = n_messages;
	double anfang = jetzt();
	while (n_messages--) {
		int ret = sendto(s,buffer,message_size,0,(struct sockaddr *) &addr,sizeof(addr));
 		if (ret == -1) fatal("sendto");
		ret = recv(s,ack,4,0);
		if (ret == -1) fatal("recv");
	}
	double ende = jetzt();
	printf("Durchsatz %lf MByte/s\n",((double) n*message_size)/1024.0/1024.0*1000.0/(ende-anfang));
}
